import { Device, EssenceMsg, GroupInfo, HonorInfo, HonorItem, HonorType, ImageInfo, MemberInfo, Message, Msg, NodeMessage, PrivateSender, QQInfo, Status, VersionInfo } from '../../Type';
import { Bot } from '../Bot';
export declare class Api {
    constructor(bot: Bot);
    private Bot;
    getApiStatus(): void;
    /**
     * 发送私聊消息
     * @param {number} user_id 对方 QQ 号
     * @param {Message} message 要发送的内容，支持纯文本和数组格式
     * @param {boolean} auto_escape 消息内容是否作为纯文本发送 ( 即不解析 CQ 码 ) , 只在 message 字段是字符串时有效
     * @returns {Promise<number>} 成功返回message_id，失败返回retcode(100)
     */
    sendPrivateMsg(user_id: number, message: Message, auto_escape?: boolean): Promise<number>;
    /**
     * 发送群消息
     * @param {number} group_id 群号
     * @param {Message} message 要发送的内容，支持纯文本和数组格式
     * @param {boolean} auto_escape 消息内容是否作为纯文本发送 ( 即不解析 CQ 码 ) , 只在 message 字段是字符串时有效
     * @returns 成功返回message_id，失败返回retcode(100)
     */
    sendGroupMsg(group_id: number, message: Message, auto_escape?: boolean): Promise<number>;
    /**
     * 发送合并转发(群)
     * @param group_id 群号
     * @param messages 自定义转发消息
     */
    sendGroupForwardMsg(group_id: number, messages: NodeMessage[]): Promise<number>;
    /**
     * 撤回信息(仅对群消息有效)
     * @param message_id 消息 ID
     */
    deleteMsg(message_id: number): void;
    /**
     * 获取消息
     * @param message_id 消息 ID
     */
    getMsg(message_id: number): Promise<Msg | undefined>;
    /**
     * 获取合并转发内容
     * @param message_id 消息 ID
     */
    getForwardMsg(message_id: number): Promise<Message[] | undefined>;
    /**
     * 获取.image文件的图片信息
     * @param file 图片缓存文件名
     */
    getImage(file: string): Promise<ImageInfo | undefined>;
    /**
     * 群组踢人
     * @param group_id 群号
     * @param user_id 要踢的 QQ 号
     * @param reject_add_request 拒绝此人的加群请求
     */
    setGroupKick(group_id: number, user_id: number, reject_add_request?: boolean): void;
    /**
     * 群组单人禁言
     * @param group_id 群号
     * @param user_id 要禁言的 QQ 号
     * @param duration 禁言时长，单位秒，0 表示取消禁言
     */
    setGroupBan(group_id: number, user_id: number, duration?: number): void;
    /**
     * 群组匿名用户禁言
     * @param group_id 群号
     * @param anonymous_flag 要禁言的匿名用户的 flag（需从群消息上报的数据中获得）
     * @param duration 禁言时长, 单位秒, 无法取消匿名用户禁言
     */
    setGroupAnonymousBan(group_id: number, anonymous_flag: string, duration?: number): void;
    /**
     * 群组全员禁言
     * @param group_id 	群号
     * @param enable 是否禁言
     */
    setGroupWholeBan(group_id: number, enable?: boolean): void;
    /**
     * 群组设置管理员
     * @param group_id 群号
     * @param user_id 要设置管理员的 QQ 号
     * @param enable true 为设置, false 为取消
     */
    setGroupAdmin(group_id: number, user_id: number, enable?: boolean): void;
    /**
     * 设置群名片（群备注）
     * @param group_id 群号
     * @param user_id 要设置的 QQ 号
     * @param card 群名片内容，不填或空字符串表示删除群名片
     */
    setGroupCard(group_id: number, user_id: number, card: string): void;
    /**
     * 设置群组名
     * @param group_id 群号
     * @param group_name 新群名
     */
    setGroupName(group_id: number, group_name: string): void;
    /**
     * 退出群组
     * @param group_id 群号
     * @param is_dismiss 是否解散，如果登录号是群主，则仅在此项为 true 时能够解散
     */
    setGroupLeave(group_id: number, is_dismiss?: boolean): void;
    /**
     * 设置群组专属头衔
     * @param group_id 群号
     * @param user_id 要设置的 QQ 号
     * @param special_title 专属头衔，不填或空字符串表示删除专属头衔
     * @param duration 专属头衔有效期，单位秒，-1 表示永久，不过此项似乎没有效果，可能是只有某些特殊的时间长度有效，有待测试
     */
    setGroupSpecialTitle(group_id: number, user_id: number, special_title: string, duration?: number): void;
    /**
     * 处理加好友请求
     * @param flag 加好友请求的 flag（需从上报的数据中获得）
     * @param approve 是否同意请求
     * @param remark 添加后的好友备注（仅在同意时有效）
     */
    setFriendAddRequest(flag: string, approve: boolean, remark: string): void;
    /**
     * 处理加群请求／邀请
     * @param flag 	加群请求的 flag（需从上报的数据中获得）
     * @param sub_type 	add 或 invite，请求类型（需要和上报消息中的 sub_type 字段相符）
     * @param approve 是否同意请求／邀请
     * @param reason 拒绝理由（仅在拒绝时有效）
     */
    setGroupAddRequest(flag: string, sub_type: 'add' | 'invite', approve: boolean, reason: string): void;
    /**
     * 获取登录号信息
     */
    getLoginInfo(): Promise<{
        /**
         * QQ 号
         */
        user_id: number;
        /**
         * QQ 昵称
         */
        nickname: string;
    }>;
    /**
     * 获取陌生人信息
     * @param user_id QQ 号
     * @param no_cache 是否不使用缓存（使用缓存可能更新不及时, 但响应更快）
     */
    getStrangerInfo(user_id: number, no_cache?: boolean): Promise<(PrivateSender & {
        qid?: number;
    }) | undefined>;
    /**
     * 获取好友列表
     */
    getFriendList(): Promise<QQInfo[]>;
    /**
     * 获取群信息
     * @param group_id 群号
     * @param no_cache 是否不使用缓存（使用缓存可能更新不及时，但响应更快）
     */
    getGroupInfo(group_id: number, no_cache?: boolean): Promise<GroupInfo | undefined>;
    /**
     * 获取群列表
     */
    getGroupList(): Promise<GroupInfo[]>;
    /**
     * 获取群成员信息
     * @param group_id 群号
     * @param user_id QQ 号
     * @param no_cache 是否不使用缓存（使用缓存可能更新不及时，但响应更快）
     */
    getGroupMemberInfo(group_id: number, user_id: number, no_cache?: boolean): Promise<MemberInfo | undefined>;
    /**
     * 获取群成员列表
     * @param group_id 群号
     */
    getGroupMemberList(group_id: number): Promise<MemberInfo[]>;
    /**
     * 获取群荣誉信息
     * @param group_id 群号
     * @param type 要获取的群荣誉类型, 可传入 talkative performer legend strong_newbie emotion 以分别获取单个类型的群荣誉数据, 或传入 all 获取所有数据
     */
    getGroupHonorInfo<T extends HonorType>(group_id: number, type: T): Promise<(HonorInfo & HonorItem[T]) | undefined>;
    /**
     * 获取版本信息
     */
    getVersionInfo(): Promise<VersionInfo | undefined>;
    /**
     * 获取运行状态
     */
    getStatus(): Promise<Status | undefined>;
    /**
     * 获取群 @全体成员 剩余次数
     * @param group_id 群号
     */
    getGroupAtAllRemain(group_id: number): Promise<{
        /**
         * 是否可以 @全体成员
         */
        can_at_all: boolean;
        /**
         * 群内所有管理当天剩余 @全体成员 次数
         */
        remain_at_all_count_for_group: number;
        /**
         * 当天剩余 @全体成员 次数
         */
        remain_at_all_count_for_uin: number;
    } | undefined>;
    /**
     * 获取当前账号在线客户端列表
     * @param no_cache 是否无视缓存
     */
    getOnlineClients(no_cache?: boolean): Promise<{
        clients: Device[];
    }>;
    /**
     * 获取群消息历史记录
     * @param group_id 群号
     * @param message_seq 起始消息序号, 可通过 get_msg 获得
     */
    getGroupMsgHistory(group_id: number, message_seq?: number): Promise<{
        messages: Message[];
    }>;
    /**
     * 设置精华消息
     * @param message_id 消息ID
     */
    setEssenceMsg(message_id: number): void;
    /**
     * 移出精华消息
     * @param message_id 消息ID
     */
    deleteEssenceMsg(message_id: number): void;
    /**
     * 获取精华消息列表
     * @param group_id 群号
     */
    getEssenceMsgList(group_id: number): Promise<EssenceMsg[]>;
    /**
     * 检查链接安全性
     * 1. 安全 2. 未知 3. 危险
     * @param url 需要检查的链接
     */
    checkUrlSafely(url: string): Promise<{
        level: 1 | 2 | 3;
    }>;
}
