"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connect = void 0;
const colors_1 = require("colors");
const websocket_1 = require("websocket");
const Tools_1 = require("../Tools");
class Connect {
    constructor(config) {
        this.wss = false;
        this.accessToken = '';
        this.host = '127.0.0.1';
        this.port = 6700;
        this.reconnection = true;
        this.reconnectionAttempts = 1000;
        this.reconnectionDelay = 1000;
        this.connectTimes = 0;
        this._messageID = 1;
        this.nextMessageID = 1;
        this.whitelist = new Set();
        this.blacklist = new Set();
        this.setNextMessage = (data, event) => {
            return (fn) => {
                const id = this.nextMessageID;
                ++this.nextMessageID;
                this.nextMessageEventList[id] = {
                    message_id: data.message_id,
                    type: event.type,
                    group_id: data['group_id'] || null,
                    user_id: data.user_id,
                    fn: async (e) => {
                        return await fn(e.message, e, data);
                    }
                };
            };
        };
        this.handleMessage = async (data) => {
            if (!data.message_type) {
                Tools_1.PrintLog.logWarning(`收到无效事件 ${colors_1.white(data.toString())}`, 'WS');
            }
            for (const event of this.messageLogEvent) {
                if (event.type === `message.${data.message_type}`) {
                    if (await event.fn(data))
                        return;
                }
            }
            for (const i in this.nextMessageEventList) {
                data.nextMessage = this.setNextMessage(data, this.nextMessageEventList[i]);
                if (this.nextMessageEventList[i].type === 'message.private') {
                    if (this.nextMessageEventList[i].user_id === data.sender.user_id) {
                        const flag = await this.nextMessageEventList[i].fn(data);
                        delete this.nextMessageEventList[i];
                        if (flag)
                            return;
                    }
                }
                else if (this.nextMessageEventList[i].type === 'message.group') {
                    if (this.nextMessageEventList[i].user_id === data.sender.user_id && this.nextMessageEventList[i].group_id === data.group_id) {
                        const flag = await this.nextMessageEventList[i].fn(data);
                        delete this.nextMessageEventList[i];
                        if (flag)
                            return;
                    }
                }
            }
            for (const event of this.messageEventList.message) {
                if (event.type === `message.${data.message_type}`) {
                    data.nextMessage = this.setNextMessage(data, event);
                    if (await event.fn(data))
                        break;
                }
            }
        };
        this.isSkip = (group_id) => {
            return (this.whitelist.size > 0 && !this.whitelist.has(group_id)) || (this.blacklist.size > 0 && this.blacklist.has(group_id));
        };
        this.connect = () => {
            if (this.connectTimes > 0) {
                Tools_1.PrintLog.logInfo(`正在尝试第${this.connectTimes}次重新链接`, 'WS');
            }
            else {
                Tools_1.PrintLog.logInfo(`正在尝试链接`, 'WS');
            }
            if (this.client) {
                this.client.close();
            }
            this.client = new websocket_1.w3cwebsocket(`${this.wss ? 'wss' : 'ws'}://${this.host}:${this.port}${this.accessToken ? '?access_token=' + this.accessToken : ''}`);
            this.client.onopen = () => {
                if (this.connectTimes > 0) {
                    Tools_1.PrintLog.logNotice(`第${this.connectTimes}次重新链接成功`, 'WS');
                }
                else {
                    Tools_1.PrintLog.logNotice('websocket连接成功', 'WS');
                }
                if (this.ready) {
                    this.ready();
                    delete this.ready;
                }
                this.connEventList.forEach(fn => {
                    fn();
                });
                this.connectTimes = 0;
                this.client.onclose = () => {
                    Tools_1.PrintLog.logWarning('websocket连接已断开', 'WS');
                    this.closeEventList.forEach(fn => {
                        fn();
                    });
                    if (this.reconnection) {
                        this.connectTimes = 1;
                        this.connect();
                    }
                };
            };
            this.client.onmessage = async (message) => {
                const data = JSON.parse(Tools_1.decode((JSON.stringify(JSON.parse(message.data.toString())))));
                if (data.group_id && this.isSkip(data.group_id))
                    return;
                if (data.post_type) {
                    if (data.post_type === 'message') {
                        this.handleMessage(data);
                    }
                    else if (data.post_type === 'notice') {
                        for (const event of this.messageEventList.notice) {
                            if (event.type === `notice.${data.notice_type}`) {
                                if (await event.fn(data))
                                    break;
                            }
                        }
                    }
                    else if (data.post_type === 'request') {
                        for (const event of this.messageEventList.request) {
                            if (event.type === `request.${data.request_type}`) {
                                if (await event.fn(data))
                                    break;
                            }
                        }
                    }
                    else if (data.post_type === 'meta_event') {
                        for (const event of this.messageEventList.meta_event) {
                            if (event.type === `meta_event.${data.meta_event_type}`) {
                                if (await event.fn(data))
                                    break;
                            }
                        }
                    }
                    else {
                        for (const event of this.messageEventList.other) {
                            if (await event.fn(data))
                                break;
                        }
                    }
                }
                if (data.echo) {
                    if (this.APIList.has(data.echo)) {
                        this.APIList.get(data.echo).fn(data);
                    }
                }
            };
            this.client.onerror = (error) => {
                if (this.connectTimes > 0) {
                    Tools_1.PrintLog.logFatal(`第${this.connectTimes}次重新链接失败`, 'WS');
                }
                else {
                    Tools_1.PrintLog.logFatal('websocket发生错误', 'WS');
                }
                this.errorEventList.forEach(fn => {
                    fn(error);
                });
                ++this.connectTimes;
                if (this.connectTimes > this.reconnectionAttempts) {
                    Tools_1.PrintLog.logError('重试次数过多', 'WS');
                }
                else {
                    setTimeout(this.connect, this.reconnectionDelay);
                }
            };
        };
        this.connEventList = [];
        this.errorEventList = [];
        this.closeEventList = [];
        this.messageEventList = {
            message: [],
            notice: [],
            request: [],
            meta_event: [],
            other: []
        };
        this.messageLogEvent = [];
        this.nextMessageEventList = {};
        this.getRes = (id, apiName, params = {}) => {
            return new Promise(resolve => {
                // let timer = setTimeout(() => {
                //   resolve({
                //     retcode: 408,
                //     status: 'failed',
                //     msg: '请求超时'
                //   })
                // }, this.timeout)
                this.APIList.set(id, {
                    info: {
                        apiName,
                        params
                    },
                    fn: (data) => {
                        resolve({
                            data: data.data,
                            retcode: data.retcode,
                            status: data.status
                        });
                        this.APIList.delete(id);
                        // clearTimeout(timer)
                        // timer = null
                    }
                });
            });
        };
        this.APIList = new Map();
        this.eventTest = (data) => {
            if (data['group_id'] && this.isSkip(data['group_id']))
                return;
            this.handleMessage(data);
        };
        if (config) {
            for (const key in config) {
                this[key] = config[key];
            }
        }
        this.connect();
    }
    get messageID() {
        return this._messageID++;
    }
    /**
     * 增加事件监听
     * message消息的log参数true时有最高优先度，同时用于控制被Ban拦截
     * 推荐使用Event类中的方法
     */
    addEvent(type, fn, log) {
        if (type === 'ws.ready') {
            this.ready = fn;
        }
        else if (type === 'ws.connect') {
            this.connEventList.push(fn);
        }
        else if (type === 'ws.error') {
            this.errorEventList.push(fn);
        }
        else if (type === 'ws.close') {
            this.closeEventList.push(fn);
        }
        else if (type.startsWith('message.')) {
            if (log) {
                this.messageLogEvent.push({
                    type,
                    fn
                });
            }
            else {
                this.messageEventList.message.push({
                    type,
                    fn
                });
            }
        }
        else if (type.startsWith('notice.')) {
            this.messageEventList.notice.push({
                type,
                fn
            });
        }
        else if (type.startsWith('request.')) {
            this.messageEventList.request.push({
                type,
                fn
            });
        }
        else if (type.startsWith('meta_event.')) {
            this.messageEventList.meta_event.push({
                type,
                fn
            });
        }
        else {
            this.messageEventList.other.push({
                type,
                fn
            });
        }
        return this;
    }
    /**
     * 获取消息监听器数量
     */
    getEventNum() {
        return this.messageEventList.message.length;
    }
    /**
     * 获取队列中未完成的消息数量
     */
    getMessageNum() {
        return this.APIList.size;
    }
    /**
     * 使用HTTP API
     * 推荐使用API类中的方法
     */
    async useAPI(apiName, params, errorLog = true) {
        if (this.isConnect()) {
            const id = this.messageID;
            this.client.send(JSON.stringify({
                action: apiName,
                params,
                echo: id
            }));
            const res = await this.getRes(id, apiName, params);
            if (errorLog && res.status !== 'ok') {
                Tools_1.PrintLog.logError(`${colors_1.white(apiName)} 调用失败 recode: ${colors_1.white(res.retcode.toString())} msg: ${colors_1.white(res.msg || '未知错误')}`, 'WS');
            }
            return res;
        }
        else {
            Tools_1.PrintLog.logError(`WebSocket连接还未建立，无法调用 ${colors_1.white(apiName)}`, 'WS');
            return { retcode: 500, status: 'failed', msg: 'WebSocket尚未链接' };
        }
    }
    /**
     * WebSocket收否已经连接
     */
    isConnect() {
        if (this.client) {
            return this.client.readyState === this.client.OPEN;
        }
        else {
            return false;
        }
    }
}
exports.Connect = Connect;
